#include <iostream>

namespace slk
{
   using std::ostream;

/**   template <typename T, typename R = T, int n = 0>
   struct number_traits
   {
      static inline
      R add(T const &left, T const &right)  {   return left + right; }
   };
*/
   // template class: n-bit integers
   // -------- ------ ----- --------

   // class forward declaration
   template <int n>
   class integer;

   // friend template function forward declarations
   template <int n>
   integer<n> operator +(integer<n>, integer<n>);
   template <int n>
   std::ostream &operator <<(ostream &, integer<n>);

   // template class definition
   template <int n>
   class integer
   {
   public:
      typedef  unsigned long u_long;

      enum {mask = (0x1 << (n - 1)) | (integer<n - 1>::mask)};

      explicit integer(u_long ul):number(ul & mask)   { ; }

      friend   integer operator +   <>(integer, integer);
      friend   ostream &operator << <>(ostream &, integer);

   private:
      u_long number;
   };

   template <>
   class integer<1>
   {
   public:
      enum {mask = 0x1};
   };

   template <int n>
   inline
   integer<n> operator +(integer<n> left, integer<n> right)
   {
      return integer<n>(left.number + right.number);
   }

   template <int n>
   inline
   std::ostream &operator <<(std::ostream &os, integer<n> in)
   {
      return os<<in.number;
   }

/**   //          ---- Gotta problem? Add up layer! ----
   template <typename T>
   inline
   T add(T const &left, T const &right)
   {
      return number_traits<T>::add(left, right);
   }
*/};

// ----------------------------------------------------------

int main(int, char *[])
{
   using slk::integer;

   using std::cout;
   using std::endl;

#ifdef __TCPLUSPLUS__
asm
{
   NOP
   NOP
   NOP
}
#endif
   integer<3> a(3);
   integer<3> b(7);

   integer<3> s = a + b;

#ifdef __TCPLUSPLUS__
asm
{
   NOP
   NOP
   NOP
}
#endif
   integer<3> d = s;

   cout << d << endl;
   return 0;
}

/*
--
 ___ __ _ _
[   ]  ] ' ] MENOS
 \   [   <   Przemysaw liwiski
[ _ ] _ ] _] menos.slk@wp.pl
*/
