//--  Befriending template functions (operators) with template class  --//

#include <iostream>

namespace slk
{
   template <typename T>
   class friend_candidate;

   template <typename T>
   void print(friend_candidate<T> const &fc)
   {
      std::cout << fc.value
                << std::endl;
   }

   template <typename T>
   class friend_candidate
   {
      T value;
   public:
      explicit friend_candidate(T const &v): value(v) {  ;  }

      // nothing really awesome!
      friend void print <>(friend_candidate<T> const &);

      // these may be more engaging...
      template <typename Ostream, typename T>
      friend Ostream &operator <<(Ostream &os, friend_candidate<T> const &fc)
      {
         return os << fc.value;
      }
      template <typename Istream, typename T>
      friend Istream &operator >>(Istream &is, friend_candidate<T> &fc)
      {
         return is >> fc.value;
      }
   };
}

int main(int, char * const [])
{
   using std::cin;
   using std::cout;
   using std::endl;

   using slk::friend_candidate;
   using slk::print;

   friend_candidate<long> fc(1000L);

   cout << fc << endl;
   cin  >> fc;
   print(fc);

   return 0;
}
