#include <algorithm>
#include <ctime>
#include <iostream>
#include <iterator>

using namespace std;

template <typename T>
struct random_gen
{
   random_gen(T seed = T())   {  srand(seed ? static_cast<size_t>(seed) : time(0)); }
   T operator ()() const      {  return rand();                                     }
};

template <typename T>
inline
int
#ifdef __BORLANDC__
_USERENTRY
#endif
cmp(void const *l, void const *r)
{
   T left  = *reinterpret_cast<const T *>(l),
     right = *reinterpret_cast<const T *>(r);
   return left < right ? -1 : (left > right ? 1 : 0);
   //return (left > right) - (left < right);
}

inline
double my_clock()
{
   return std::clock()/static_cast<double>(CLOCKS_PER_SEC);
}

int main(int, char * const[])
{
   size_t size = 1500000;
   double *array = new double[size];

   generate(array, array + size,
            random_gen<double>(10));

   double start = my_clock();
   sort(array, array + size);
   cout << "std::sort: " << my_clock() - start << endl;

   generate(array, array + size,
            random_gen<double>(10));

   start = my_clock();
   qsort(array, size, sizeof(double), cmp<double>);
   cout << "qsort:     " << my_clock() - start << endl;

   delete []array;
   return 0;
}
