#pragma hdrstop
#include <algorithm>
#include <iostream>
#include <string>

namespace slk
{
   // forward declarations
   template <typename Char_t>
   class Text;

   // character plenipotentiary
   template <typename Char_t>
   class char_ref
   {
      int           n;
      Text<Char_t> &r;
   public:
      char_ref(Text<Char_t> &_r, int const _n): r(_r), n(_n)                        {  ;  }
      Char_t &operator =(char_ref const &cr) {  return this->operator =(cr.r.txt[cr.n]);  }
      Char_t &operator =(Char_t const &c)    {  return r._copy(), r.txt[n] = c;           }
      operator Char_t () const               {  return r.txt[n];                          }
   };

   // a wannabe std::string class
   template <typename Char_t>
   class Text
   {
      bool   shared;
      Char_t  *txt;

      void _copy();
   public:
      Text(): shared(true), txt(0)                       {  ;  }
      Text(Char_t * const s): shared(true), txt(s)       {  ;  }
      Text(Text const &s):  shared(true), txt(s.txt)     {  ;  }
      Text &operator =(Text const &s)
      {
         if(!shared) delete txt;
         shared = true;
         txt = s.txt;

         return *this;
      }
      char_ref<Char_t> operator [](int n) {  return char_ref<Char_t>(*this, n);  }
      Char_t &operator [](int n) const    {  return txt[n];                      }
      // Leon
      ~Text()                             {  if(!shared) delete txt;             }

      // befriending
      template <typename Ostream>
      friend Ostream &operator <<(Ostream &os, Text<Char_t> const &tc)
      {
         return os << tc.txt;
      }
      friend class char_ref<Char_t>;
   };

   template <typename Char_t>
   void Text<Char_t>::_copy()
   {
      using std::basic_string;
      using std::copy;

      if(shared)
      {
         basic_string<Char_t> wstr = txt;

         size_t  size = wstr.size() + 1;
         Char_t *tmp = new Char_t[size];

         copy(wstr.begin(), wstr.end(), tmp);

         txt   = tmp;
         shared = false;
      }
   }

   template <>
   void Text<char>::_copy()
   {
      using std::strcpy;
      using std::strlen;

      if(shared)
      {
         char *tmp = new char[strlen(txt) + 1];
         strcpy(tmp, txt);
         txt = tmp;
         shared = false;
      }
   }
}

int main(int, wchar_t *[])
{
   using std::endl;
   using std::cout;
   using slk::Text;

   char  mercedes[] = "Mercedes  320 CDI";

   slk::Text<char> A_class = mercedes;
   slk::Text<char> C_class = A_class;
   slk::Text<char> E_class = C_class;

   slk::Text<char> S_class;
                   S_class = E_class;

   A_class[9] = 'A';
   A_class[9] =  A_class[9];
   C_class[9] = 'C';
   E_class[9] = 'E';
   S_class[9] = 'S';

   cout << S_class[9] << endl
        << S_class    << endl

        << E_class[9] << endl
        << E_class    << endl
        << C_class[9] << endl
        << C_class    << endl

        << A_class[9] << endl
        << A_class    << endl;
   return 0;
}
