#include <cmath>
#include <iostream>
#include <iomanip>
#include <list>
#include <sstream>
/**
   Program generates the simple (i.e. with only '1' in numerators) continued
   fraction representation of the number X

                     X =    1
                         -------
                         A0 +  1
                             -----
                             A1 + ...

   in LaTeX fashion, i.e. as:

         $X=\frac{1}{A_{0}+\frac{1}{A_{1}+\cdots }}$

   TODO: 1. Taking into consideration X > 1 (easy, but tediuos).
         2. Do it recursively (done, but without '\cdots\ yet) - see list.

   It is not to much for it to compute $\frac{10946}{17711}$
   which has the representation:

   \frac{10946}{17711}=
   \frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}
   {1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}
   {1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}{1+\frac{1}{2
   }}}}}}}}}}}}}}}}}}}}

   */
using namespace std;
//---------------------------------------------------------------------------

template <typename number_type>
class number2scf
{
   number_type const integer,
                     decimal;
   list<number_type> scf;
   size_t            depth;
public:
   number2scf(number_type number, size_t _d = 15):integer(floor(number)),
                                                  decimal(number - integer),
                                                  depth(_d)
   {  ;  }
};

template <typename number_type, typename char_type>
class scf2latex;

double const eps = 1.0e-7;

int main(int, char * const argv[])
{
   double N = atof(argv[1]),
          D = atof(argv[2]);

//   double const N = 3.1415926535897932384626433832795,
//                D = 1;
//              $\tau \sim
                N = 0.21132486540518711774542560974902;

   size_t order = atoi(argv[3]);

   // LaTeX header
   cout << setprecision(18)
        << "\\documentclass[10pt]{article}\n"
        << "\\begin{document}\n\n";

   // LaTeX body
   ostringstream N_D;
   N_D << setprecision(18);

   if(D == 1) N_D << N;
   else       N_D << "\\frac{" << N << "}{" << D << "}";

   cout << "The simple continued fraction of $"
        << N_D.str() << "$ is\n"
        << "\\[\n"
        << N_D.str() << "=\n";

   ostringstream scf;
   double X = N/D;
   if(X > 1.0)
   {
      scf << floor(X) << "+";
      X -= floor(X);
   }
   string curls;
   long offset = scf.tellp();
   while(X > eps && order)
   {
      // compute next factor of the fraction
      double const A = floor(1.0/X);

      // print the LaTeX fraction chain term
      scf << "\\frac{1}";
      bool linebreak = (scf.tellp() - offset) > 68
                       &&
                       (scf.tellp() - offset) % 80 > 68;
      scf << (linebreak ? "\n" : "")
          << "{" << A << "+";

      // compute new X
      X = 1.0/X - A;
      // collect curly brackets
      curls += "}";
      // countdown
      --order;
   }
   string frac = scf.str();
   if(!order)
   {
      // add dots since there is no exact representation
      frac += "\\cdots";
   }
   else
   {
      // we obtained the exact representation. Hence, remove last '+'
      frac.erase(frac.find_last_of('+'));
   }
   cout << frac + "\n" + curls;

   // LaTeX's rest of the body and footer
   cout << "\n\\]\n"
        << "\\end{document}\n";

   return 0;
}
//---------------------------------------------------------------------------


/**
{1+\frac{1}
#include <exception>
#include <list>
size_t scf(double x, list<double> &ld, size_t counter)
{
   if(counter && x > eps)
   {
      double a = floor(1.0L/x);
             x = 1.0L/x - a;
      ld.push_back(a);

      --counter;
      scf(x, ld, counter);
   }
   return counter;
}

void adjust(list<double> &ld)
{
   list<double>::iterator first = ld.begin(),
                          last  = ld.end();
   // check emptiness
   if(first != last)
   {
      list<double>::iterator one_before_last = last;
      --one_before_last;

      if(one_before_last != first)
      {
         if(*one_before_last == 1)
         {
             one_before_last != first ? --one_before_last : one_before_last;
            *one_before_last += 1;
         }
         ++one_before_last;
         ld.erase(one_before_last, ld.end());
      }
   }
}

list<double>::iterator export2tex(list<double>::iterator first,
                                  list<double>::iterator last)
{
   list<double>::iterator one_before_last = last;
                        --one_before_last;
   if(first != one_before_last)
   {
      cout << "\\frac{1}{" << *first << "+";
      first++;
      export2tex(first, last);
   }
   else
   {
      cout << "\\frac{1}{" << *first;
   }
   cout << "}";
   return first;
}

   list<double> ld;

   double _D = static_cast<double>(D);
   scf(N/_D, ld, 10), adjust(ld), export2tex(ld.begin(), ld.end());

   // LaTeX's rest of the body and footer
   cout << "\n\\]\n"
        << "\\end{document}\n";
*/


