#include <ctime>
#include <iostream>

namespace slk
{
   template <int start, int stop, int step = 1>
   class for_loop
   {
      enum { go = (start + step < stop) };
   public:
      static inline int perform(/* arguments for loop statement */)
      {
         // put your for_loop code here
         return for_loop<go * (start + step), go * stop, go * step>::perform();
      }
   };

   template <>
   class for_loop<0, 0, 0>
   {
   public:
      static inline int perform(/* arguments for loop statement */)
      {
         // do nothing
         return 0;
      }
   };
}

/**
* Three diferent implementations of bit_reversing for FFT;
* 1. Full CT
*/
namespace CT
{
   template <int bits, int n>
   struct bit_reverse
   {
      enum {  t = ((1 & n) << bits - 1) | bit_reverse<bits - 1, n >> 1>::t};
   };

   template <int n>
   struct bit_reverse<0, n>
   {
      enum {  t = 1 & n };
   };
}
/**
* 2. Full RT
*/
namespace RT
{
   template <typename T, size_t bits>
   struct bit_reverse
   {
      static size_t reverse(T n, size_t b = bits);
   };

   template <typename T, size_t bits>
   size_t bit_reverse<T, bits>::reverse(T n, size_t b)
   {
      T t = T();
      while(b--)
      {
         t <<= 1;
         t  |= 1 & n;
         n >>= 1;
      }
      return t;
   }
}
/**
* 3. Unrolled in CT, performed in RT...
*/
namespace CT_RT
{
   template <typename T, size_t bits>
   struct bit_reverse
   {
      static inline T reverse(T const n)
      {
         T t = (1 & n) << bits - 1;
         return t | bit_reverse<T, bits - 1>::reverse(n >> 1);
      }
   };

   template <typename T>
   struct bit_reverse<T, 0>
   {
      static inline T reverse(T const n)
      {
         return 0;
      }
   };
}

inline
double my_clock()
{
   return std::clock()/static_cast<double>(CLK_TCK);
}

int main(int, char * const[])
{
   using std::cout;
   using std::endl;
   using std::time;

   typedef RT::bit_reverse<size_t, 32>      RTBR;
   typedef CT_RT::bit_reverse<size_t, 32> CTRTBR;
/**
*   A little bit of Monte-Carlo method...

   using   std::rand;

   for(size_t i = rand(); RTBR::reverse(i) == CTRTBR::reverse(i); i = rand());
   cout << "F...!\n";
*/

   double start = my_clock();
   size_t sum   = 0,
          size  = 0x0fffffff;
   // RT
   for(size_t i = 0; i < size; ++i)
   {
      sum ^= RTBR::reverse(i);
   }

   double stop = my_clock();
   cout << sum          << endl
        << stop - start << endl;
   // CT_RT
   start = my_clock();
   sum   = 0;
   for(size_t i = 0; i < size; ++i)
   {
      sum ^= CTRTBR::reverse(i);
   }
   stop = my_clock();
   cout << sum          << endl
        << stop - start << endl;
   return 0;
}
